/*
 * GooCanvas Demo. Copyright (C) 2006 Damon Chaplin.
 * Released under the GNU LGPL license. See COPYING for details.
 *
 * demo-item.c - a simple demo item.
 */
#include <math.h>

#include "goocanvas.h"
#include "demo-item.h"

/* Use the GLib convenience macro to define the type. GooDemoItem is the
   class struct, goo_demo_item is the function prefix, and our class is a
   subclass of GOO_TYPE_CANVAS_ITEM_SIMPLE. */
G_DEFINE_TYPE (GooDemoItem, goo_demo_item, GOO_TYPE_CANVAS_ITEM_SIMPLE)

/* The standard object initialization function. */
    static void
goo_demo_item_init (GooDemoItem *demo_item)
{
    demo_item->size = 0.0;
}

void
goo_demo_item_move_to (GooCanvasItem	*item,
		       gdouble		 x,
		       gdouble		 y)
{
    cairo_matrix_t matrix;

    cairo_matrix_init_translate (&matrix, x, y);
    goo_canvas_item_set_transform (item, &matrix);
}

/* The convenience function to create new items. This should start with a 
   parent argument and end with a variable list of object properties to fit
   in with the standard canvas items. */
GooCanvasItem*
goo_demo_item_new (GooCanvasItem      *parent,
		   gdouble             x,
		   gdouble             y,
		   gdouble             size,
		   char                letter,
		   ...)
{
    GooCanvasItem *item;
    GooDemoItem *demo_item;
    const char *first_property;
    va_list var_args;

    item = g_object_new (GOO_TYPE_DEMO_ITEM, NULL);

    demo_item = (GooDemoItem*) item;
    demo_item->size = size;
    demo_item->letter = letter;

    va_start (var_args, letter);
    first_property = va_arg (var_args, char*);
    if (first_property)
	g_object_set_valist ((GObject*) item, first_property, var_args);
    va_end (var_args);

    if (parent)
    {
	goo_canvas_item_add_child (parent, item, -1);
	g_object_unref (item);
    }

    goo_demo_item_move_to (item, x, y);

    return item;
}

/* The update method. This is called when the canvas is initially shown and
   also whenever the object is updated and needs to change its size and/or
   shape. It should calculate its new bounds, storing them in simple->bounds,
   and it should convert these to device coordinates. */
static void
goo_demo_item_update  (GooCanvasItemSimple *simple,
		       cairo_t             *cr)
{
    GooDemoItem *demo_item = (GooDemoItem*) simple;

    /* Compute the new bounds. */
    simple->bounds.x1 = 0;
    simple->bounds.y1 = 0;
    simple->bounds.x2 = demo_item->size;
    simple->bounds.y2 = demo_item->size;

    /* Convert to device coordinates. */
    goo_canvas_item_simple_user_bounds_to_device (simple, cr, &simple->bounds);
}

/* The paint method. This should draw the item on the given cairo_t, using
   the item's own coordinate space. */
static void
goo_demo_item_paint (GooCanvasItemSimple *simple,
		     cairo_t             *cr,
		     GooCanvasBounds     *bounds)
{
    GooDemoItem *item = (GooDemoItem*) simple;
    cairo_pattern_t *gradient;
    cairo_text_extents_t extents;
    int rad = (int) (item->size / 2);
    int cx = item->size / 2;
    int cy = cx;
    int tx, ty;
    double spot_angle = M_PI / 4.0;
    double spot_rad = rad / 2.0;
    char string[2];

    cairo_save (cr);

    gradient = cairo_pattern_create_radial (cx - spot_rad * cos (spot_angle),
					    cy - spot_rad * sin (spot_angle),
					    0.0,
					    cx - spot_rad * cos (spot_angle),
					    cy - spot_rad * sin (spot_angle),
					    rad + spot_rad);
    cairo_pattern_add_color_stop_rgb (gradient, 0.0, 1.0, 1.0, 1.0);
    cairo_pattern_add_color_stop_rgb (gradient, 1.0, 0.33, 0.33, 0.33);

    cairo_set_source (cr, gradient);

    cairo_arc (cr,
	       cx, cy,
	       rad, 0, 2 * M_PI);

    cairo_fill (cr);

    cairo_select_font_face (cr, "mono",
			    CAIRO_FONT_SLANT_NORMAL,
			    CAIRO_FONT_WEIGHT_BOLD);
    cairo_set_font_size (cr, 1.8 * rad);

    string[0] = item->letter;
    string[1] = '\0';
    cairo_text_extents (cr, string, &extents);
    tx = cx - extents.width / 2 - extents.x_bearing;
    ty = cy - extents.height / 2 - extents.y_bearing;

    cairo_set_source_rgb (cr, 0.7, 0.7, 0.7);
    cairo_move_to (cr, tx + 1, ty + 1);
    cairo_show_text (cr, string);
		
    cairo_set_source_rgb (cr, 0.33, 0.33, 0.33);
    cairo_move_to (cr, tx - 1, ty - 1);
    cairo_show_text (cr, string);

    cairo_set_source_rgb (cr, 0.2, 0.3, 0.8);
    cairo_move_to (cr, tx, ty);
    cairo_show_text (cr, string);

    cairo_restore (cr);
}

/* Hit detection. This should check if the given coordinate (in the item's
   coordinate space) is within the item. If it is it should return the item,
   otherwise it should return NULL. */
static GooCanvasItem*
goo_demo_item_get_item_at (GooCanvasItemSimple *simple,
			   gdouble              x,
			   gdouble              y,
			   cairo_t             *cr,
			   gboolean             is_pointer_event)
{
    GooDemoItem *demo_item = (GooDemoItem*) simple;

    if (x < 0 || (x > demo_item->size)
	|| y < 0 || (y > demo_item->size))
	return NULL;

    return (GooCanvasItem*) simple;
}

/* The class initialization function. Here we set the class' update(), paint()
   and get_item_at() methods. */
static void
goo_demo_item_class_init (GooDemoItemClass *klass)
{
    GooCanvasItemSimpleClass *simple_class = (GooCanvasItemSimpleClass*) klass;

    simple_class->update        = goo_demo_item_update;
    simple_class->paint         = goo_demo_item_paint;
    simple_class->get_item_at   = goo_demo_item_get_item_at;
}
